import graph;
// This file contains a series of useful asymptote functions.
// Defines normal distribution
real normalf(real mean=0.0, real sigma=1.0, real x) {return 1/(sqrt(2pi)*sigma)*exp(-.5((x-mean)/sigma)^2);}
// Defines binomial distribution
real binomf(int n, real p, int k) {return choose(n,k)*p^k*(1-p)^(n-k);};
// The following are convenience drawing function for working with
// discrete probability distributions.
//
// Draws a bar of width one at integer point k of height h(k).
// Outline color is drawpen.  The fill colour is fillpen which defaults
// to invisible (i.e. not shown).
void drawbar(int k, real h(int), pen drawpen=black, pen fillpen=invisible)
{
  path g=box((k-.5,0),(k+.5,h(k)));
  draw(g,drawpen);
  filldraw(g,fillpen); 
}
// Draws a range of bars, from integer low to high.  See drawbar for other
// options.
void drawbarrange(real h(int), int low, int high, pen drawpen=black, pen fillpen=invisible)
{
  for(int i=low; i <= high; ++i) {
    drawbar(i,h,drawpen,fillpen);
  }
}
// Draws discrete distribution at points (x,h(x)) where x ranges from integer
// low to high.  Default is to just draw dots whose color and size are
// determined by dotpen and sizedot respectively.  If linepen is set a
// vertical line is drawn from the point to the x-axis.  Default is for
// linepen to be invisible.
void drawdiscreterange(real h(int), int low, int high, pen dotpen=black, int sizedot=7, pen linepen=invisible)
{
  pair F(int x) {return (x,h(x));}
  dotfactor=sizedot;
  for(int i=low; i <= high; ++i) {
    draw((i,0)..F(i), linepen);
    dot(F(i), dotpen);
  }
}
void drawfunc(real f(real), real low, real high, pen drawpen=black+1, pen fillpen=invisible)
{
  path g=graph(f,low,high,operator --)--(high,0)--(low,0)--cycle;
  fill(g,p=fillpen);
  draw(graph(f,low,high),drawpen);
}
// A function to trivially return x=t.  Needed for plotting x=g(y) below
// which uses parameterization.
real identfunc(real t) {return t;}
// The following draws x=f(y) along the y-axis.
void ydrawfunc(real f(real), real low, real high, pen drawpen=black+1, pen fillpen=invisible)
{
  path g=graph(f,identfunc,low,high,operator --)--(0,high)--(0,low)--cycle;
  fill(g,p=fillpen);
  draw(graph(f,identfunc,low,high),drawpen);
}
// A function to represent the y-axis
real yaxisfunc(real x) {return 0.0;}
// Draws a bar of width dx over interval [a+(i-1)dx, a+(i)dx] for integer i.
// The bar has upper limit h(a+(i-1)dx+shift)) and lower limit
// l(a+(i-1)dx+shift)).  If l is not given the lower limit is the y-axis.
// If shift is zero then these functions are evaluated at the left endpoint.
// Choosing shift=.5 or 1.0 give bars evaluated at the midpoint and
// right endpoints respectively.
// Outline color is drawpen.  The fill colour is fillpen which defaults
// to invisible (i.e. not shown).
void drawcontinuousbar(int i, real a, real dx, real shift=0.0, real h(real), pen drawpen=black, pen fillpen=invisible, real l(real)=yaxisfunc)
{
  // path g=box((k-.5,0),(k+.5,h(k)));
  path g=box((a+(i-1)*dx,l(a+(i-1+shift)*dx)),(a+i*dx,h(a+(i-1+shift)*dx)));
  draw(g,drawpen);
  filldraw(g,fillpen); 
}
// Draws a range of n bars, from real a to b.  See drawcontinuousbar for other
// variables.
void drawcontinuousbarrange(real h(real), int n, real a, real b, real shift=0.0, pen drawpen=black, pen fillpen=invisible, real l(real)=yaxisfunc)
{
  for(int i=1; i <= n; ++i) {
    drawcontinuousbar(i, a, (b-a)/n, shift, h, drawpen, fillpen, l);
  }
}
